package com.keyware.sonar.java.rules.checkers;


import org.sonar.check.Rule;
import org.sonar.java.ast.visitors.SubscriptionVisitor;
import org.sonar.plugins.java.api.tree.*;
import org.springframework.expression.EvaluationException;

import java.lang.reflect.Method;
import java.sql.ClientInfoStatus;
import java.util.Collections;
import java.util.List;


@Rule(key = "DynamicCodeChecker")

//检测代码中包含动态代码执行操作时，工具进行提示
public class DynamicCodeChecker extends SubscriptionVisitor {

    @Override
    public List<Tree.Kind> nodesToVisit() {
        return Collections.singletonList(
                Tree.Kind.METHOD_INVOCATION
        );
    }

    @Override
    public void visitNode(Tree tree) {
        MethodInvocationTree node = (MethodInvocationTree) tree;
        System.out.println(node);
        var expressionTree = node.methodSelect();
        if (expressionTree instanceof MemberSelectExpressionTree) {
            var exprTree = (MemberSelectExpressionTree) expressionTree;
            var name = exprTree.identifier();
            if ("eval".equals(name.toString())) {
                var varNameNode = exprTree.expression();
                if (varNameNode instanceof IdentifierTree) {
                    var varName = (IdentifierTree) varNameNode;
                    var symbol = varName.symbol();
                    var varDecler = symbol.declaration();
                    if (varDecler != null) {
                        var variableTree = (VariableTree) varDecler;
                        var typeName = variableTree.type().toString();
                        if ("ScriptEngine".equals(typeName)) {
                            context.reportIssue(this, tree, "程序设计时禁止动态构建代码进行功能实现");
                        }
                    }
                }
            }
        }
    }

}
